########################################
#
# This program retrieves PISA data from 
# http://www.ebi.ac.uk/msd-srv/prot_int/cgi-bin/piserver
#
# The results are written to output_<filename>.csv
#
# echung@spkaa.com July 11, 2013 using Python 2.7.5
#
########################################

import urllib, urllib2, time, re, sys, os

timedelay = 1  #IMPORTANT! This is the delay (secs) between URL
                #requests to avoid getting blocked by the server. 

########################################
# MAIN FUNCTION
########################################
if len(sys.argv) != 2:
    print "Syntax: python.exe batch_pisa_v2.py <pdb_list.txt>"
    sys.exit(-1)

########################################
# This paragraph reads the csv or txt file
# into a python list pdb_list[]
########################################
pdb_list = []
try:
    infile = open(sys.argv[1], "r")
    filebuffer = infile.readlines()
    for line in filebuffer:
        if line:
            #Append to list adds each item to the list 
            pdb_list.append(line.strip().lower())
    infile.close()
except:
    print "Error opening "+sys.argv[1]
    sys.exit(-1)
print "pdb_list = "+str(pdb_list)

#
#Check if file exists, rename if found
#
if os.path.exists("output_"+sys.argv[1]+".csv"):
    os.rename("output_"+sys.argv[1]+".csv","output_"+sys.argv[1]+"_"+str(time.time())+".csv")

########################################
# This paragraph is the "magic", calls PISA site
########################################
count=0
for item in pdb_list:
    count = count+1
    
    #
    #Build URL call, refer to URLLIB and URLLIB2 documentation"
    #
    url = "http://www.ebi.ac.uk/msd-srv/pisa/cgi-bin/piserver"
    values = { 'page_key' : 'sform_page', 
              'action_key' : 'act_submit_pdb', 
              'dir_key' : '632-4B-2AF', 
              'session_point' : '2', 
              'radio_sf_session_type' : 'id_sf_dbsearch', 
              'radio_source' : 'id_pdbentry', 
              'entry0' : item, 
              'entry1' : item, 
              'edt_pdbcode' : item }
    post = urllib.urlencode(values)
    request = urllib2.Request(url,post)
    response = urllib2.urlopen(request)
    html = response.read()
    
    #
    #Use REGULAR EXPRESSION (REGEX) to find the text
    #and values that we want to record
    #
    search = re.findall(r'Analysis: <strong><input style="color:blue;bgcolor:white; border:0;" size="64" id="analtext" name="analtext" value="(.*)"/></strong></big>\n</td>\n</tr>',html)
    if len(search)==1:
        analysis = search[0]
        print item +":\tAnalysis:\t\t"+analysis
    else:
        analysis = "???"
        print item+": \t*Error finding Analysis for "+item
        
    search = re.findall(r'Most probable assembly:</font>&nbsp;<sup><a href="javascript:openWindow.*A href="javascript:JSViewAssembly\(\)">(.*)</A></td>\n</tr>',html)
    if len(search)==1:
        assembly = search[0]
        print item +":\tMost probable assembly:\t"+ assembly
    elif "structures appear to be monomeric" in html:
        assembly = "1-mer"
        print item +":\tMost probable assembly:\t"+ assembly
    else:
        assembly = "???"
        print item+": \t*Error finding Most Probable Assembly for "+item
    
    #REGEX to pull digits from answers above
    search = re.findall(r'(\d+) amino acid chain',analysis)
    if len(search)==1:
        chains = search[0]
    else:
        chains = ""
    
    search = re.findall(r'(\d+) ligand.* in ASU',analysis)
    if len(search)==1:
        ligands = search[0]
    else:
        ligands = ""
    
    search = re.findall(r'(\d+)-mer',assembly)
    if len(search)==1:
        mer = search[0]
    else:
        mer = ""
    
    #
    #This paragraph opens a csv file and appends the data
    #found above one line at a time.
    #    
    outfile = open("output_"+str(sys.argv[1])+".csv","a")
    outfile.write(item+",")
    outfile.write(analysis+",")
    outfile.write(str(chains)+","+str(ligands)+",")
    outfile.write(assembly+",")
    outfile.write(str(mer)+"\n")
    outfile.close()
    
    #Wait 1 second before calling website again
    time.sleep(timedelay)

#END FOR LOOP

print "\n"+"END PROGRAM NORMAL. DATA SAVED TO output_"+sys.argv[1]+".csv"
print "SEARCHED "+str(count)+" ITEMS."